/testing/guestbin/swan-prep
ipsec start
../../guestbin/wait-until-pluto-started
ipsec whack --impair add-unknown-v2-payload-to:IKE_SA_INIT
ipsec auto --add westnet-eastnet-ipv4-psk-ikev2
echo "initdone"
