.axis path, .axis line {
  fill: none;
  stroke: #000;
  shape-rendering: crispEdges;
}

.line {
  fill: none;
  stroke: steelblue;
  stroke-width: 1px;
}

/* applied to circles AND text */

.passed {
    fill: green;
}
.failed {
    fill: orange;
}
.unresolved {
    fill: red;
}
.untested {
    fill: blue;
}

/* applied to circles only */
.passed > circle.current {
    fill: white;
    stroke: green;
}
.failed > circle.current {
    fill: white;
    stroke: orange;
}
.unresolved > circle.current {
    fill: white;
    stroke: red;
}
.untested > circle.current {
    fill: white;
    stroke: blue;
}

text.current {
    fill: brown;
}

.empty {
    fill: white;
    stroke: brown;
}
