/testing/guestbin/swan-prep
ipsec _stackmanager start
ifconfig eth1:1 192.1.2.46 netmask 255.255.255.0
ifconfig eth1:2 192.1.2.47 netmask 255.255.255.0
ifconfig eth1:3 192.1.2.48 netmask 255.255.255.0
ifconfig eth1:4 192.1.2.49 netmask 255.255.255.0
ifconfig eth1:5 192.1.2.50 netmask 255.255.255.0
ifconfig eth1:6 192.1.2.51 netmask 255.255.255.0
echo "initdone"
