# New files should have dropped in, and we are ready to restart
ipsec restart
../../guestbin/wait-until-pluto-started
ipsec status | grep 192.1.2
# connection is ondemand, but we will --up so we can see X.509 auth
# name is generated by System Role
ipsec auto --up 192.1.2.45-to-192.1.2.23
ping -n -q -c 4 -I 192.1.2.45 192.1.2.23
# show non-zero IPsec traffic counters
ipsec whack --trafficstatus
echo done
