# Two pices of dogma at play:

# - when ever an impair code path gets executed it should be clearly
#   logged

# - *.console.txt should only contain useful stuff

# only used because the responder is being a little slow.
/: IMPAIR: suppressing retransmits; scheduling timeout in/ d
