#!/bin/sh

. ../../default-testparams.sh
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ip-xfrm-policy-mobike.sed"
REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ip-xfrm-state-mobike.sed"
ROAD_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS strongswan.sed"
